//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>
#include <time.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qdom.h>
#include <qdir.h>

//Projectwide includes
#include "album.h"
#include "subalbum.h"
#include "photo.h"
#include "imageTools.h"
#include "xmlTools.h"
#include "md5.h"
#include "../gui/saveDialog.h"
#include "../gui/loadDialog.h"
#include "../gui/subalbumPreviewWidget.h"
#include "../config.h"

#include <iostream.h>
//==============================================
///Sets default information
Album::Album()
{
  //set strings to default values
  name = "";
  description ="";
  author = "";

  //by default no representative image
  smallRepresentativeImage = new QPixmap( QString(IMAGE_PATH)+"notSpecified.png" );
  largeRepresentativeImage = NULL;
  
  //no Subalbums by default
  firstSubalbum = NULL;
  lastSubalbum = NULL;

  //set last modification date to today
  updateModificationDate();
  
  //no subalbums
  numSubalbums = 0;
  numLoadedSubalbums = 0;
  
  //not previously saved by default
  savedToDisk = false;
  
  //construct temp directory to save images to
  QDir rootDir( QDir::homeDirPath() );
  if(!rootDir.exists( ".albumShaper" ))
  {
    rootDir.mkdir( ".albumShaper" );
  }
  rootDir.cd( ".albumShaper" );
  if(!rootDir.exists( "tmp" ))
  {
    rootDir.mkdir( "tmp" );
  }
  saveLocation = QDir::homeDirPath() + "/.albumShaper/tmp";  
}
//==============================================
///Frees Subalbums
Album::~Album()    
{
  //delete representative image
  delete smallRepresentativeImage;
  delete largeRepresentativeImage;
  
  //delete subalbums  
  Subalbum* current = firstSubalbum;
  Subalbum* temp;
  while(current != NULL)
  {
    temp = current->getNext();
    delete current;
    current = temp;
  }
} 
//==============================================
///Sets the album name
void Album::setName(QString val)
{
  name = val;
}
//==============================================
///Gets the album name
QString Album::getName()
{
  return QString(name);
}
//==============================================
///Sets the album description
void Album::setDescription(QString val)
{
  description = val;
}
//==============================================
///Gets the album description
QString Album::getDescription()
{
  return QString(description);
}
//==============================================
///Sets the album author
void Album::setAuthor(QString val)
{
  author = val;
}
//==============================================
///Gets the album author
QString Album::getAuthor()
{
  return QString(author);
}
//==============================================
///Sets the representative image
void Album::setRepresentativeImages(QImage* rawImage)
{
  //---------------------------------------------------------    
  //delete representative images
  delete smallRepresentativeImage;
  delete largeRepresentativeImage;
  //---------------------------------------------------------    
  //compute representative image sizes
  int smallRepWidth = 0;
  int smallRepHeight = 0;
  int largeRepWidth = 0;
  int largeRepHeight = 0;
  resizeImage( rawImage->width(), rawImage->height(),
               107, 80,
               smallRepWidth, smallRepHeight);
  resizeImage( rawImage->width(), rawImage->height(),
               500, 320,
               largeRepWidth, largeRepHeight);
  //---------------------------------------------------------    
  //create various representative images
  //---------------------------------------------------------    
  //copy and scale small version
  QImage thumbnailSmall = rawImage->smoothScale( smallRepWidth, smallRepHeight );
  thumbnailSmall.setAlphaBuffer(true);
  smallRepresentativeImage = new QPixmap( smallRepWidth, smallRepHeight );
  smallRepresentativeImage->convertFromImage( thumbnailSmall );  

  //copy and scale large version
  QImage thumbnailLarge = rawImage->smoothScale( largeRepWidth, largeRepHeight );
  thumbnailLarge.setAlphaBuffer(true);
  largeRepresentativeImage = new QPixmap( largeRepWidth, largeRepHeight );
  largeRepresentativeImage->convertFromImage( thumbnailLarge );  
}
//==============================================
///Returns the representative image
QPixmap* Album::getRepresentativeImage(int size)
{
  if(size == SMALL)
    return smallRepresentativeImage;
  else if(size == LARGE)
    return largeRepresentativeImage;
  else
    return NULL;
}
//==============================================
///Returns a pointer to the first Subalbum
Subalbum* Album::getFirstSubalbum()
{
  return firstSubalbum;
}
//==============================================
///Returns a pointer to the last Subalbum
Subalbum* Album::getLastSubalbum()
{
  return lastSubalbum;
}
//==============================================
void Album::appendSubalbum(Subalbum* val)
{
  numSubalbums++;
  
  //first subalbum
  if(firstSubalbum == NULL)
  {
    firstSubalbum = val;
    lastSubalbum = val;
  }
  //append to end of list
  else
  {
    lastSubalbum->setNext( val );
    lastSubalbum = val;
  }
}
//==============================================
void Album::removeSubalbum(Subalbum* val)
{
  //search through linked list and remove item
  Subalbum* prev = NULL;
  Subalbum* current = firstSubalbum;
  while(current != NULL)
  {
    //found it! remove it
    if(current == val)
    { 
      //update first pointer
      if(firstSubalbum == val)
        firstSubalbum = val->getNext();
      
      //update last pointer
      if(lastSubalbum == val) 
        lastSubalbum = prev;

      //update next pointer
      if(prev != NULL)
        prev->setNext( current->getNext() );
            
      //delete object
      delete val;
      val = NULL;
      numSubalbums--;
      return;
    }
    
    prev = current;
    current = current->getNext();
  }

}
//==============================================
///Returns the last modified year
int Album::getModificationYear()
{
  return modificationYear;
}
//==============================================
///Returns the last modified month
int Album::getModificationMonth()
{
  return modificationMonth;
}
//==============================================
///Returns the last modified day
int Album::getModificationDay()
{
  return modificationDay;
}
//==============================================
///Updates the modification date to today's date
void Album::updateModificationDate()
{
  //set last modification date to today
  struct tm *newtime;
  time_t aclock;

  //get time in seconds
  time( &aclock ); 

  //convert to tm form
  newtime = localtime( &aclock );

  //set year, month, and day
  modificationYear = newtime->tm_year + 1900;
  modificationMonth = newtime->tm_mon + 1;
  modificationDay = newtime->tm_mday;
}
//==============================================
void Album::importFromDisk(LoadDialog* dialog, QString fileName)
{
  dialog->printMessage("Parsing XML");
  
  //open file
  QFile albumFile( fileName );
  if( !albumFile.open( IO_ReadOnly ) )
  {
    cout << "Error! Unable to open file!\n";
    return;
  }
  
  //parse dom
  QDomDocument albumDom;
  if( !albumDom.setContent( &albumFile ) )
  {
    cout << "Error! Unable to construct DOM!\n";
    return;  
  }

  //close file
  albumFile.close();

  //get main directory all other files and subdirectories are in  
  QString rootDir = QFileInfo(albumFile).dirPath(TRUE);
  saveLocation = rootDir + "/img";
  
  
  //if representative image exists load
  QImage repImage(rootDir + "/img/album.jpg");
  if(!repImage.isNull())
  {
    setRepresentativeImages(&repImage);  
  }  

  int subalbumNum = 0;
  
  //get root node and start parsing DOM
  QDomElement root = albumDom.documentElement();
  QDomNode node = root.firstChild();
  QDomText val;  
  while( !node.isNull() )
  {
    //------------------------------------------------------------
    //album name
    if( node.isElement() && node.nodeName() == "name" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        name = val.nodeValue();
    }
    //------------------------------------------------------------
    //album description
    else if( node.isElement() && node.nodeName() == "description" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        description = val.nodeValue();
    }
    //------------------------------------------------------------
    //album author
    else if( node.isElement() && node.nodeName() == "author" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        author = val.nodeValue();
    }
    //------------------------------------------------------------
    //album modification date
    else if( node.isElement() && node.nodeName() == "modified" )
    {     
      QDomNode childNode = node.firstChild();
      bool ok;
      int intVal;
      while( !childNode.isNull() )
      {
        //------------------------------------------------------------
        if( childNode.isElement() && childNode.nodeName() == "year" )
        {
          val = childNode.firstChild().toText();
          if(!val.isNull())
          {  
            intVal = val.nodeValue().toInt( &ok );
            if(ok)
              modificationYear = intVal;
            else
              cout << "Error parsing modification year!\n";
          }
        }        
        //------------------------------------------------------------
        else if( childNode.isElement() && childNode.nodeName() == "month" )
        {
          val = childNode.firstChild().toText();
          if(!val.isNull())
          {
            intVal = val.nodeValue().toInt( &ok );
            if(ok)
              modificationMonth = intVal;
            else
              cout << "Error parsing modification month!\n";
          }
        }        
        //------------------------------------------------------------
        else if( childNode.isElement() && childNode.nodeName() == "day" )
        {
          val = childNode.firstChild().toText();
          if(!val.isNull())
          {
            intVal = val.nodeValue().toInt( &ok );
            if(ok)
              modificationDay = intVal;
            else
              cout << "Error parsing modification day!\n";
          }
        }        
        //------------------------------------------------------------
        //advance to next node   
        childNode = childNode.nextSibling();
        //------------------------------------------------------------
      }
    }
    //------------------------------------------------------------
    //subalbum
    else if( node.isElement() && node.nodeName() == "subalbum" )
    {
      //increase counter    
      subalbumNum++;

      //create new subalbum
      Subalbum* salbum = new Subalbum(this, numSubalbums+1);
      
      //populate it
      salbum->importFromDisk( &node, subalbumNum, dialog, (rootDir + "/") );
      
      //append it to list of subalbums
      appendSubalbum(salbum);
    }
    //------------------------------------------------------------
    //advance to next node   
    node = node.nextSibling();
    //------------------------------------------------------------
  }

  //reset number of loaded subalbums  
  numLoadedSubalbums = numSubalbums;

  dialog->printMessage("Done!");
   
  //save load directory name and loaded/saved bit
  saveLocation = rootDir;
  savedToDisk = true;
}
//==============================================
void Album::exportToDisk(SaveDialog* dialog, QString dirName)
{
  saveLocation = dirName;
  exportToDisk(dialog, true);
}
//==============================================
void Album::exportToDisk(SaveDialog* dialog, bool forceSave)
{
  //------------------------------------------
  //create subdirs
  QDir localDir(saveLocation);
  //img dirs
  localDir.mkdir("img");
  //subalbum dirs
  localDir.setPath(saveLocation + "/img");
  int i;
  for(i=1; i <= numSubalbums; i++)
  {
    QString dirName = QString("%1") .arg(i);
    localDir.mkdir(dirName);
  }
  //------------------------------------------    
  //checks worked, go ahead with export
  exportSublabumsToHTML(dialog);
  exportTopLevelImages();
  exportSubalbumImages(dialog, forceSave);
  reorderSubalbumImages(dialog);
  removeStagnantImages(dialog);
  exportToXML(dialog);
  exportToHTML(dialog);
  //------------------------------------------
  //remove files from temp folder
  QDir tmpDir(QDir::homeDirPath() + "/.albumShaper/tmp" );
  QStringList strLst = tmpDir.entryList();
  QStringList::iterator it;
  for(it = strLst.begin(); it != strLst.end(); it++)
  {
      tmpDir.remove(*it);
  }
  //------------------------------------------
  //print done message
  savedToDisk = true;
  dialog->printMessage("Done!");
  //------------------------------------------
}
//==============================================
void Album::exportToXML(SaveDialog* dialog)
{
  //print dialog message
  dialog->printMessage("XML Layout");
  
  //update modification date
  updateModificationDate();
  
  //create/open xml file
  QFile* xml = new QFile(saveLocation + "/Album.xml");
  if(xml->open(IO_WriteOnly))
  {
    QTextStream stream( xml );
   
    //write album information
    stream << "<album>\n";
    stream << "  <name>" << fixXMLString(name) << "</name>\n";
    stream << "  <description>" << fixXMLString(description) << "</description>\n";
    stream << "  <author>" << fixXMLString(author) << "</author>\n";  
    stream << "  <modified>\n";
    stream << "    <year>" << modificationYear << "</year>\n";
    stream << "    <month>" << modificationMonth << "</month>\n";
    stream << "    <day>" << modificationDay << "</day>\n";
    stream << "  </modified>\n";
   
    //write subalbums
    Subalbum* current = firstSubalbum;
    while(current != NULL)
    {
      current->exportToXML(stream);
      current = current->getNext();
    }
  
    //end album
    stream << "</album>\n";    
    xml->close(); 
  }
  else
    cout << "error opening file!\n";
}
//==============================================
void Album::exportToHTML(SaveDialog* dialog)
{
  //print dialog message
  dialog->printMessage("Album HTML");
  
  //update modification date
  updateModificationDate();
  
  //create/open html file
  QFile* html = new QFile(saveLocation + "/Album.html");
  if(html->open(IO_WriteOnly))
  {
    QTextStream stream( html );
    stream << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";    
    stream << "<html>\n";
    stream << "  <head>\n";
    stream << "    <style type=\"text/html\">\n";
    stream << "      <!--\n";
    stream << "      A{text-decoration:none}\n";
    stream << "      -->\n";
    stream << "    </style>\n";
    stream << "    <style type=\"text/css\">\n";
    stream << "     <!--\n";
    stream << "     h3 {font-size: 12pt; font-weight: bold; text-align: center}\n";
    stream << "     h4 {font-size: 10pt; font-weight: normal; text-align: center}\n";
    stream << "     h4.ital {font-size: 10pt; font-weight: normal; text-align: center}\n";
    stream << "      -->\n";
    stream << "    </style>\n";    
    stream << "    <title>" << name << "</title>\n";
    stream << "    <meta name=\"generator\" content=\"Album Shaper (c.) Will Stokes\">\n";
    stream << "  </head>\n";
    stream << "  <body>\n";
    
    if(getRepresentativeImage(LARGE) != NULL)
    {
      stream << "    <center>\n";
      stream << "      <img src=\"img/album.jpg\" alt=\"Representative Album Image\">\n";
      stream << "    </center>\n";
    }
    stream << "    <center>\n";
    stream << "      <table>\n";
    stream << "        <tr>\n";
    if(largeRepresentativeImage != NULL)
    {
      stream << "          <td width=\"" << largeRepresentativeImage->width() << "\">\n";
    }
    else    
    {
      stream << "          <td width=\"450\">\n";
    }
    stream << "            <h3>\n";
    stream << "              " << name << "\n";
    stream << "            </h3>\n";
    stream << "            <h4>\n";
    stream << "              " << description << "\n";
    stream << "            </h4>\n";
    stream << "          </td>\n";
    stream << "        </tr>\n";
    stream << "      </table>\n";
    stream << "    </center>\n";
    
    int subalbumNumber = 0;
    
    //write subalbum thumbnails
    Subalbum* current = firstSubalbum;
    if(current != NULL)
    {
      stream << "    <center>\n";
      stream << "      <table border=\"0\">\n";
      while(current != NULL)
      {
        Subalbum* backupCurrent = current;
        int backupNumber = subalbumNumber;
        
        //---------------------------------------
        //Print row of images
        //---------------------------------------
        int n = 3;
        stream << "        <tr>\n";
        while(current != NULL && n > 0)
        {
          n--;
          subalbumNumber++;          
          stream << "          <td valign=\"bottom\" width=\"" << THUMBNAIL_WIDTH << "\">\n";
          if(current->getRepresentativeImage(LARGE) != NULL )
          {
            stream << "            <center>\n";
            stream << "              <script type=\"text/javascript\" language=\"JavaScript\">\n";
            stream << "              <!-- HIDE FROM OLD BROWSERS\n";
            stream << "              document.write(\"<a href=\\\"subalbum_" << subalbumNumber << "_slideshow.html\\\">\")\n";
            stream << "              document.write(\"  <img alt=\\\"Subalbum " << subalbumNumber << "\\\" src=\\\"img/" << subalbumNumber << "_thumb.jpg\\\">\")\n";
            stream << "              document.write(\"</a>\")\n";
            stream << "              -->\n";
            stream << "              </script>\n";
            stream << "              <noscript>\n";
            stream << "                <a href=\"subalbum_" << subalbumNumber << "_thumbs.html\">\n";
            stream << "                  <img alt=\"Subalbum " << subalbumNumber << "\" src=\"img/" << subalbumNumber << "_thumb.jpg\">\n";
            stream << "                </a>\n";
            stream << "              </noscript>\n";
            stream << "            </center>\n";
          }
          stream << "          </td>\n";
          current = current->getNext();
        }
        while(n != 0 && subalbumNumber > 2)
        {
          stream << "          <td width=\"" << THUMBNAIL_WIDTH << "\"></td>\n";
          n--;
        }
        //---------------------------------------
        //Print row of image descriptions
        //---------------------------------------
        current = backupCurrent;
        subalbumNumber = backupNumber;
        n = 3;
        stream << "        <tr>\n";
        while(current != NULL && n > 0)
        {          
          subalbumNumber++;
          stream << "          <td valign=\"top\" width=\"" << THUMBNAIL_WIDTH << "\">\n";
          stream << "            <center>\n";
          stream << "              <script type=\"text/javascript\" language=\"JavaScript\">\n";
          stream << "              <!-- HIDE FROM OLD BROWSERS\n";
          stream << "              document.write(\"<a href=\\\"subalbum_" << subalbumNumber << "_slideshow.html\\\">\")\n";
          stream << "              document.write(\"  " << current->getName() << "\")\n";
          stream << "              document.write(\"</a>\")\n";
          stream << "              -->\n";
          stream << "              </script>\n";
          stream << "              <noscript>\n";
          stream << "                <a href=\"subalbum_" << subalbumNumber << "_thumbs.html\">\n";
          stream << "                  " << current->getName() << "\n";
          stream << "                </a>\n";
          stream << "              </noscript>\n";
          stream << "            </center>\n";
          stream << "          </td>\n";
          n--;
          current = current->getNext();
        }
        
        while(n != 0 && subalbumNumber > 2)
        {
          stream << "          <td></td>\n";
          n--;
        }
      }
      
      stream << "      </table>\n";
      stream << "    </center>\n";
    } //end if
  
    stream << "    <h4 class=\"ital\">\n";
    stream << "      (Generated for " << author << " on " << modificationMonth << "/" << modificationDay;
    stream << "/" << modificationYear << " by <a href=\"http://albumshaper.sourceforge.net\">Album Shaper</a>)\n";
    stream << "    </h4>\n";
    stream << "  </body>\n";
    stream << "</html>\n";
      
    html->close(); 
  }
  else
    cout << "error opening file!\n";
}
//==============================================
void Album::exportSublabumsToHTML(SaveDialog* dialog)
{
  int n=0;
  Subalbum* current = firstSubalbum;
  while(current != NULL)
  {
    n++;    
    
    //print dialog message
    dialog->printSubalbumHTML( n );
    
    //export thumbnail album
    QString fileName = QString(saveLocation + "/subalbum_%1_thumbs.html") .arg(n);
    QFile* thumbHTML = new QFile(fileName);
    if(thumbHTML->open(IO_WriteOnly))
    {
      QTextStream stream( thumbHTML );
      current->exportThumbnailHTML( stream, n );
      thumbHTML->close(); 
    }
    
    //export slideshow
    fileName = QString(saveLocation + "/subalbum_%1_slideshow.html") .arg(n);
    QFile* slideshowHTML = new QFile(fileName);
    if(slideshowHTML->open(IO_WriteOnly))
    {
      QTextStream stream( slideshowHTML );
      current->exportSlideshowHTML( stream, n );
      slideshowHTML->close(); 
    }
    
    //move on to next subalbum
    current = current->getNext();  
  }  
}
//==============================================
void Album::exportTopLevelImages()
{
  //if image set export it
  if(getRepresentativeImage(LARGE) != NULL)
  {
    getRepresentativeImage(LARGE)->save(saveLocation + "/img/album.jpg", "JPEG", 100);
  }
  //else make sure any previously set images are removed
  else
  {
    QDir rootDir(saveLocation + "/img/");
    rootDir.remove("album.jpg");
  }
  
  //export subalbum thumbs
  int n=0;
  Subalbum* current = firstSubalbum;
  while(current != NULL)
  {
    n++;
    //if subalbum has representative image export it
    if(current->getRepresentativeImage(LARGE) != NULL )
    {
      QString fileName = QString(saveLocation + "/img/%1_thumb.jpg" ).arg(n);
      current->getRepresentativeImage(LARGE)->save(fileName, "JPEG", 100);
    }
    //otherwise make sure anyprevious set images are removed
    else
    {
      QDir rootDir(saveLocation + "/img/");
      rootDir.remove( QString("%1_thumb.jpg").arg(n) );
    }
    current = current->getNext();
  }  
}
//==============================================
void Album::exportSubalbumImages(SaveDialog* dialog, bool forceSave)
{  
  dialog->printMessage("Saving images");

  //iterate over all subalbums
  int subalbumNumber=0;
  Subalbum* currentSubalbum = firstSubalbum;
  while(currentSubalbum != NULL)
  {
    subalbumNumber++;    
    
    //iterate over all photos in this subalbum
    int photoNumber=0;    
    Photo* currentPhoto = currentSubalbum->getFirst();
    while(currentPhoto != NULL)
    {
      photoNumber++;
      dialog->printSubalbumPhoto( subalbumNumber, photoNumber );
      //---------------------------------------
      //if the current photo does not need to be saved then move on
      if( !forceSave && !currentPhoto->getNeedsSavingVal() )
      {
        currentPhoto = currentPhoto->getNext();
        continue;
      }
      //---------------------------------------
      //construct a QDir object for the tmp folder
      QDir rootDir;
      //---------------------------------------
      //get initial photo # and subalbum #, used for saving      
      int initPhotoNumber = currentPhoto->getInitialPhotoNumber();
      int initSubalbumNumber = currentPhoto->getInitialSubalbumNumber();
      //---------------------------------------
      //export thumbnail image
      QString fileName = QString(saveLocation + "/img/%1/%2_thumb.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      currentPhoto->getImage(THUMBNAIL)->save(fileName, "JPEG", 100);

      //compute and store md5 for thumbnail     
      ifstream thumbnailFile(fileName.ascii());
      if(thumbnailFile.is_open())
      {
        currentPhoto->setThumbnailChecksum( getMD5(thumbnailFile) ); 
        thumbnailFile.close();
      }
      //---------------------------------------
      //export slideshow image
      QString oldName = currentPhoto->getSlideshowFilename();
      QString newName = QString(saveLocation + "/img/%1/%2_slideshow.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);

      //if file has been modified move from current location to final location
      if( currentPhoto->getNeedsSavingVal() )
      {
        //attempt to rename file, if fails do explicit copy, then delete prev file
         if(!rootDir.rename( oldName, newName))
         {
           copyFile(oldName, newName);
           rootDir.remove(oldName);
         }
      }
      //If file has not been modified we must be doing a save-as and saving has been forced. In this case
      //COPY file from current location to final location, DON'T delete previous copy!!!
       else
       {
         copyFile(oldName, newName);
       }
             
      //compute and store md5 for slideshow image
      ifstream file(newName.ascii());
      if(file.is_open())
      {
        currentPhoto->setSlideshowChecksum( getMD5(file) ); 
        file.close();
      }
      //---------------------------------------
      //export full size image
      oldName = currentPhoto->getImageFilename();   
      newName = QString(saveLocation + "/img/%1/%2.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      
      //if file has been modified move from current location to final location
      if( currentPhoto->getNeedsSavingVal() )
      {
        //attempt to rename file, if fails do explicit copy, then delete prev file
         if(!rootDir.rename( oldName, newName))
         {
           copyFile(oldName, newName);
           rootDir.remove(oldName);
         }
      }
      //If file has not been modified we must be doing a save-as and saving has been forced. In this case
      //COPY file from current location to final location, DON'T delete previous copy!!!
       else
       {
         copyFile(oldName, newName);
       }
            
      //compute and store md5 for image
      ifstream imageFile(newName.ascii());
      if(imageFile.is_open())
      {
        currentPhoto->setImageChecksum( getMD5(imageFile) ); 
        imageFile.close();
      }
      //---------------------------------------
      //set new storage locations of files
      currentPhoto->setImageFilename( QString(saveLocation + "/img/%1/%2.jpg").arg(initSubalbumNumber).arg(initPhotoNumber) );
      currentPhoto->setSlideshowFilename( QString(saveLocation + "/img/%1/%2_slideshow.jpg").arg(initSubalbumNumber).arg(initPhotoNumber) );
      //---------------------------------------
      //set image as not needing saving
      currentPhoto->setNeedsSavingVal(false);
      //---------------------------------------
      //move on to next photo in subalbum
      currentPhoto = currentPhoto->getNext();
      //---------------------------------------
    }
    //---------------------------------------
    //move on to next subalbum
    currentSubalbum = currentSubalbum->getNext();  
  }
}
//==============================================
void Album::reorderSubalbumImages(SaveDialog* dialog)
{
  //--------------------------------------------------------
  //--------------------------------------------------------
  //first pass over all photos, those whose initial and current numbers don't match up
  //rename slightly so we don't overwrte them the second time around
  //--------------------------------------------------------
  //--------------------------------------------------------
  dialog->printMessage("Reordering images (pass 1)");
  
  //iterate over all subalbums
  int subalbumNumber=0;
  Subalbum* currentSubalbum = firstSubalbum;
  while(currentSubalbum != NULL)
  {
    subalbumNumber++;    
    
    //iterate over all photos in this subalbum
    int photoNumber=0;    
    Photo* currentPhoto = currentSubalbum->getFirst();
    while(currentPhoto != NULL)
    {
      photoNumber++;
      dialog->printSubalbumPhoto( subalbumNumber, photoNumber );
      //---------------------------------------
      //get initial photo # and subalbum #, used for checks and moving
      int initPhotoNumber = currentPhoto->getInitialPhotoNumber();
      int initSubalbumNumber = currentPhoto->getInitialSubalbumNumber();
      //---------------------------------------
      //if the current photo has not moved then move on
      if( initPhotoNumber == photoNumber &&
          initSubalbumNumber == subalbumNumber)
      {
        currentPhoto = currentPhoto->getNext();
        continue;
      }
      //---------------------------------------
      //rename full image, slideshow image, and thumbnail image     
      QDir rootDir;
      QString oldName = QString(saveLocation + "/img/%1/%2.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      QString newName = QString(saveLocation + "/img/%1/%2_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);     
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //-----
      oldName = QString(saveLocation + "/img/%1/%2_slideshow.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      newName = QString(saveLocation + "/img/%1/%2_slideshow_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);      
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //-----
      oldName = QString(saveLocation + "/img/%1/%2_thumb.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      newName = QString(saveLocation + "/img/%1/%2_thumb_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);      
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //---------------------------------------
      //move on to next photo in subalbum
      currentPhoto = currentPhoto->getNext();
      //---------------------------------------
    }
    //---------------------------------------
    //move on to next subalbum
    currentSubalbum = currentSubalbum->getNext();  
  }

  //--------------------------------------------------------
  //--------------------------------------------------------
  //second pass over all photos, those whose initial and current numbers don't match up
  //rename to their final names and reset initial photo and subalbum numbers
  //--------------------------------------------------------
  //--------------------------------------------------------
  dialog->printMessage("Reordering images (pass 2)");
  
  //iterate over all subalbums
  subalbumNumber=0;
  currentSubalbum = firstSubalbum;
  while(currentSubalbum != NULL)
  {
    subalbumNumber++;    
    
    //iterate over all photos in this subalbum
    int photoNumber=0;    
    Photo* currentPhoto = currentSubalbum->getFirst();
    while(currentPhoto != NULL)
    {
      photoNumber++;
      dialog->printSubalbumPhoto( subalbumNumber, photoNumber );
      //---------------------------------------
      //get initial photo # and subalbum #, used for checks and moving
      int initPhotoNumber = currentPhoto->getInitialPhotoNumber();
      int initSubalbumNumber = currentPhoto->getInitialSubalbumNumber();
      //---------------------------------------
      //if the current photo has not moved then move on
      if( initPhotoNumber == photoNumber &&
          initSubalbumNumber == subalbumNumber)
      {
        currentPhoto = currentPhoto->getNext();
        continue;
      }
      //---------------------------------------
      //rename full image, slideshow image, and thumbnail image to their final names
      QDir rootDir;
      QString oldName = QString(saveLocation + "/img/%1/%2_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      QString newName = QString(saveLocation + "/img/%1/%2.jpg" ).arg(subalbumNumber).arg(photoNumber);     
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //-----
      oldName = QString(saveLocation + "/img/%1/%2_slideshow_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      newName = QString(saveLocation + "/img/%1/%2_slideshow.jpg" ).arg(subalbumNumber).arg(photoNumber);      
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //-----
      oldName = QString(saveLocation + "/img/%1/%2_thumb_moved.jpg" ).arg(initSubalbumNumber).arg(initPhotoNumber);
      newName = QString(saveLocation + "/img/%1/%2_thumb.jpg" ).arg(subalbumNumber).arg(photoNumber);      
      if(!rootDir.rename( oldName, newName))
      {
        QImage(oldName).save(newName, "JPEG", 100);
        rootDir.remove( oldName );
      }
      //---------------------------------------
      //reset initial photo and subalbum numbers
      currentPhoto->setInitialPhotoNumber(photoNumber);
      currentPhoto->setInitialSubalbumNumber(subalbumNumber);
      //---------------------------------------
      //move on to next photo in subalbum
      currentPhoto = currentPhoto->getNext();
      //---------------------------------------
    }
    //---------------------------------------
    //move on to next subalbum
    currentSubalbum = currentSubalbum->getNext();  
  }
}
//==============================================
void Album::removeStagnantImages(SaveDialog* dialog)
{
  dialog->printMessage("Removing stagnant images");
  QDir rootDir(saveLocation + "/img/");
  //---------------------------------
  //remove stagnant photos in existing subalbums
  int subalbumNumber=0;
  Subalbum* currentSubalbum = firstSubalbum;
  while(currentSubalbum != NULL)
  {
    subalbumNumber++;    

    int i;
    for(i=currentSubalbum->getNumPhotos() + 1;
        i <= currentSubalbum->getNumLoadedPhotos();
        i++)
    {
      QString thumbString = QString(saveLocation + "/img/%1/%2_thumb.jpg" ).arg(subalbumNumber).arg(i);
      QString slideshowString = QString(saveLocation + "/img/%1/%2_slideshow.jpg" ).arg(subalbumNumber).arg(i);
      QString imageString = QString(saveLocation + "/img/%1/%2.jpg" ).arg(subalbumNumber).arg(i);
      rootDir.remove(thumbString);
      rootDir.remove(slideshowString);
      rootDir.remove(imageString);
    }
    
    //reset number of loaded photos since old photos removed now
    currentSubalbum->resetNumLoadedPhotos();
    
    //move on to next subalbum
    currentSubalbum = currentSubalbum->getNext();  
  }
  //---------------------------------
  //remove stagnant subalbums and all their contents
  int i;
  for(i=numSubalbums+1; i<=numLoadedSubalbums; i++)
  {
    //get filelist for directory
    QDir imageDir(  QString(saveLocation + "/img/%1/").arg(i) );
    QStringList list = imageDir.entryList( QDir::Files );
    
    //remove each file in directory
    for ( QStringList::Iterator file = list.begin(); file != list.end(); ++file ) 
    {
      rootDir.remove( QString(saveLocation + "/img/%1/" + *file).arg(i) );
    }
    
    //remove directory
    rootDir.rmdir( QString("%1").arg(i) );
    
    //remove thumbnail image
    rootDir.remove( QString(saveLocation + "/img/%1_thumb.jpg").arg(i) );
    
    //remove slideshow and thumbnail html pages
    rootDir.remove( QString(saveLocation + "/subalbum_%1_thumbs.html").arg(i) );    
    rootDir.remove( QString(saveLocation + "/subalbum_%1_slideshow.html").arg(i) );        
  }
  
  //reset number of loaded subalbums since stagnant directories removed now
  numLoadedSubalbums = numSubalbums;
  //---------------------------------
}  
//==============================================
bool Album::prevSave()
{
  return savedToDisk;
}
//==============================================
void Album::syncSubalbumList(SubalbumPreviewWidget* item)
{
  //base case, no items
  if(item == NULL)
  {
    firstSubalbum = NULL;
    lastSubalbum = NULL;
    return;
  }
  
  //set first and last pointers
  firstSubalbum = item->getSubalbum();
  firstSubalbum->setNext(NULL);
  lastSubalbum = firstSubalbum;
    
  //set all next pointers
  while(item->nextItem() != NULL)
  {
    item->getSubalbum()->setNext( ((SubalbumPreviewWidget*)item->nextItem())->getSubalbum() );
    item = (SubalbumPreviewWidget*)item->nextItem();
    lastSubalbum = item->getSubalbum();
    lastSubalbum->setNext(NULL);
  }
}
//==============================================
QString Album::getSaveLocation()
{
  return saveLocation;
}
//==============================================
int Album::getNumSubalbums()
{
  return numSubalbums;
}
//==============================================
