//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qdom.h>
#include <qdir.h>

//Projectwide includes
#include "photo.h"
#include "subalbum.h"
#include "imageTools.h"
#include "xmlTools.h"
#include "../config.h"

#include <iostream.h>

//==============================================
///Sets default information
Photo::Photo(Subalbum* subalbum, int number)
{
  //set subalbum pointer
  this->subalbum = subalbum;
  
  //set initial photo number and subalbum number
  initialNumber = number;
  initialSubalbumNumber = subalbum->getSubalbumNumber();
  
  //set strings to default values
  description ="";

  //set next pointer to NULL
  next = NULL;
  
  //set thumbnail image
  thumbnailImage = NULL;
  paddedThumbnailImage = NULL; 
  slideshowImage = NULL;
  fullImage = NULL;
  
  //set checksums to the empty string by default
  imageChecksum = "";
  slideshowChecksum = "";
  thumbnailChecksum = "";
  
  //image does not need to be saved by default
  needsSaving = false;
  
  //filenames not set by default
  imageLocation = "";
  slideshowLocation = "";
  
  //no dimension of slideshow image known yet
  slideshowWidth = -1;
  slideshowHeight = -1;
}
//==============================================
///Destructor
Photo::~Photo()
{
  if(paddedThumbnailImage != thumbnailImage)
    delete paddedThumbnailImage;
  delete thumbnailImage;
  delete slideshowImage;
  delete fullImage;
} 
//==============================================
bool Photo::setImage(QString filename)
{
  //reset image object
  delete fullImage;
  fullImage = new QImage(filename);
  if(fullImage->isNull()) return false;

  //create slideshow and thumbnail formats
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 
  
  //store image and slideshow filenames
  imageLocation = filename;  

  //image is being stored in temp location, needs saving!
  needsSaving = true;
  return true;
}
//==============================================
bool Photo::setImage(QString imageName,
                              QString slideshowName,
                              QString thumbnailName)
{
  //---------------------------------------------------------    
   //free old images
  delete fullImage;
  if(paddedThumbnailImage != thumbnailImage)
    delete paddedThumbnailImage;
  delete thumbnailImage;
  delete slideshowImage;
  
  fullImage = NULL;
  paddedThumbnailImage = NULL;
  thumbnailImage = NULL;
  slideshowImage = NULL;
  //---------------------------------------------------------    
  //construct thumbnail image
  thumbnailImage = new QImage(thumbnailName);
  if(thumbnailImage->isNull()) return false;
  //---------------------------------------------------------    
  //construct padded thumbnail image
  //if scaled thumbnail image is same size as padded thumbnail should be reuse object
  if(thumbnailImage->width() == THUMBNAIL_WIDTH &&
     thumbnailImage->height() == THUMBNAIL_HEIGHT)
  {  
    paddedThumbnailImage = thumbnailImage;  
  }
  //else pad thumbnail
  else
  {
    //created padded thumbnail image
    paddedThumbnailImage = new QImage(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT, thumbnailImage->depth());
    paddedThumbnailImage->setAlphaBuffer(true);
  
    //copy scaled image into centered portion of padded image
    int xDiff = THUMBNAIL_WIDTH - thumbnailImage->width();
    int yDiff = THUMBNAIL_HEIGHT - thumbnailImage->height();

    //set all pixels to white
    int x,y;
    for(x=0; x< THUMBNAIL_WIDTH; x++)
    {
      for(y=0; y<THUMBNAIL_HEIGHT; y++)
      {
        paddedThumbnailImage->setPixel(x, y, QColor(255, 255, 255).rgb()); 
      }
    }
  
    int x2 = 0;
    int y2;
    for(x = xDiff/2; x< (xDiff/2) + thumbnailImage->width(); x++)
    {
      y2 = 0;
      for(y = yDiff/2; y < (yDiff/2) + thumbnailImage->height(); y++)
      {
        paddedThumbnailImage->setPixel(x, y, thumbnailImage->pixel(x2, y2));
        y2++;
      }
      x2++;  
    }
  }  
  //---------------------------------------------------------    
  //save new image filenames
  imageLocation = imageName;  
  slideshowLocation = slideshowName;
  //---------------------------------------------------------    
  //image loaded from file, no need to save it again just yet
  needsSaving = false;  
  return true;
  //---------------------------------------------------------    
}
//==============================================
bool Photo::setImage(QImage* newFullImage)
{
  //reset image object
  delete fullImage;
  fullImage = newFullImage;
  if(fullImage->isNull()) return false;

  //create slideshow and thumbnail formats
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 

  //save out updated full and slideshow images to temp folder and deallocate memory
  imageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
  slideshowLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
         
  fullImage->save( imageLocation, "JPEG", 100);
  slideshowImage->save( slideshowLocation, "JPEG", 100);
 
  //mark image as having been modified and hence in the tmp folder  
  needsSaving = true;  

  //deallocate qimage objects
  deallocateLargeImages();
  
  return true;
}
//==============================================
void Photo::setDescription(QString val)
{
  description = val;
}
//==============================================
///Gets the description
QString Photo::getDescription()
{
  return QString(description);
}
//==============================================
///Gets the next photo
Photo* Photo::getNext()
{
  return next;
}
//==============================================
///Sets the next photo
void Photo::setNext(Photo* val)
{
  next = val;
}
//==============================================
QImage* Photo::getImage(int size)
{
  if(size == THUMBNAIL)
    return thumbnailImage;
  else if(size == PADDED_THUMBNAIL)
    return paddedThumbnailImage;
  else if(size == SLIDESHOW)
  {
    if( slideshowImage != NULL)
      return slideshowImage;
    else
    {
      if(!needsSaving)
      {
        return new QImage(slideshowLocation);
      }
      else
      {
        QString tmpImageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                                .arg(initialSubalbumNumber)
                                                                .arg(initialNumber);
         return new QImage(tmpImageLocation);
      }
    }
  }
  else if(size == IMAGE)
  {
    if( fullImage != NULL)
      return fullImage;
    else 
    {
      if(!needsSaving)
      {
        return new QImage(imageLocation);
      }
      else
      {
        QString tmpImageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                                .arg(initialSubalbumNumber)
                                                                .arg(initialNumber);
         return new QImage(tmpImageLocation);
      }
    }
  }
  else
    return NULL;
}
//==============================================
void Photo::exportToXML(QTextStream& stream)
{
  //write photo information
  stream << "    <photo>\n";
  stream << "      <description>" << fixXMLString(description) << "</description>\n";
  stream << "      <imageMD5>" << fixXMLString(imageChecksum) << "</imageMD5>\n";
  stream << "      <slideMD5>" << fixXMLString(slideshowChecksum) << "</slideMD5>\n";
  stream << "      <thumbMD5>" << fixXMLString(thumbnailChecksum) << "</thumbMD5>\n";
  stream << "    </photo>\n";
}
//==============================================
void Photo::exportThumbnailHTML(QTextStream& stream)
{
  //write photo information
  stream << "?";
}
//==============================================
void Photo::exportSlideshowHTML(QTextStream& stream)
{
  //write photo information
  stream << "?";
}
//==============================================
void Photo::importFromDisk(QDomNode* root)
{
  QDomNode node = root->firstChild();
  QDomText val;  
  while( !node.isNull() )
  {
    //------------------------------------------------------------
    //photo description
    if( node.isElement() && node.nodeName() == "description" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        description = val.nodeValue();
    }
    //------------------------------------------------------------
    //image md5
    if( node.isElement() && node.nodeName() == "imageMD5" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        imageChecksum = val.nodeValue();
    }
    //------------------------------------------------------------
    //slideshow md5
    if( node.isElement() && node.nodeName() == "slideMD5" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        slideshowChecksum = val.nodeValue();
    }
    //------------------------------------------------------------
    //thumbnail md5
    if( node.isElement() && node.nodeName() == "thumbMD5" )
    { 
      val = node.firstChild().toText();
      if(!val.isNull())
        thumbnailChecksum = val.nodeValue();
    }
    //------------------------------------------------------------
    //advance to next node   
    node = node.nextSibling();
    //------------------------------------------------------------
  }
}
//==============================================
void Photo::rotate90()
{
  //load up image
  //if not modified load from permanent location
  fullImage = new QImage(imageLocation);
  
  //create new image
  QImage* rotatedImage = new QImage(fullImage->height(),
                                    fullImage->width(),
                                    fullImage->depth());
  rotatedImage->setAlphaBuffer(true);
  //copy data                                    
  int x,y;
  for(x=0; x < fullImage->height(); x++)
  {
    for(y=0; y < fullImage->width(); y++)
    {
      rotatedImage->setPixel(fullImage->height() - 1 - x, y, fullImage->pixel(y, x) );
    }
  }                                    

  //delete old image and set new pointer
  delete fullImage;
  fullImage = rotatedImage;
  
  //create thumbnail
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 
  
  //save out updated full and slideshow images to temp folder and deallocate memory
  imageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
  slideshowLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
         
  fullImage->save( imageLocation, "JPEG", 100);
  slideshowImage->save( slideshowLocation, "JPEG", 100);
 
  //mark image as having been modified and hence in the tmp folder  
  needsSaving = true;  

  //deallocate qimage objects
  deallocateLargeImages();
  
  //update slideshow width/height
  int temp = slideshowWidth;
  slideshowWidth = slideshowHeight;
  slideshowHeight = temp;
}
//==============================================
void Photo::rotate270()
{
  //load up image
  //if not modified load from permanent location
  fullImage = new QImage(imageLocation);
  
  //create new image
  QImage* rotatedImage = new QImage(fullImage->height(),
                                    fullImage->width(),
                                    fullImage->depth());
  rotatedImage->setAlphaBuffer(true);
  //copy data                                    
  int x,y;
  for(x=0; x < fullImage->height(); x++)
  {
    for(y=0; y < fullImage->width(); y++)
    {
      rotatedImage->setPixel(x, fullImage->width() - 1 - y, fullImage->pixel(y, x) );
    }
  }                                    

  //delete old image and set new pointer
  delete fullImage;
  fullImage = rotatedImage;
  
  //create thumbnail
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 
  
  //save out updated full and slideshow images to temp folder and deallocate memory
  imageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
  slideshowLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
         
  fullImage->save( imageLocation, "JPEG", 100);
  slideshowImage->save( slideshowLocation, "JPEG", 100);
 

 //mark image as having been modified and hence in the tmp folder  
 needsSaving = true;  

 //deallocate qimage objects
 deallocateLargeImages();
  
  //update slideshow width/height
  int temp = slideshowWidth;
  slideshowWidth = slideshowHeight;
  slideshowHeight = temp;
}
//==============================================
void Photo::flipHorizontally()
{
  //load up image
  //if not modified load from permanent location
  fullImage = new QImage(imageLocation);
  
  //create new image
  QImage* rotatedImage = new QImage(fullImage->width(),
                                    fullImage->height(),
                                    fullImage->depth());
  rotatedImage->setAlphaBuffer(true);
  //copy data                                   
  int x,y; 
  for(x=0; x < fullImage->width(); x++)
  {
    for(y=0; y < fullImage->height(); y++)
    {
      rotatedImage->setPixel(x, fullImage->height() - 1 - y, fullImage->pixel(x, y) );
    }
  }                                    

  //delete old image and set new pointer
  delete fullImage;
  fullImage = rotatedImage;
  
  //create thumbnail
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 
  
  //save out updated full and slideshow images to temp folder and deallocate memory
  imageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
  slideshowLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
         
  fullImage->save( imageLocation, "JPEG", 100);
  slideshowImage->save( slideshowLocation, "JPEG", 100); 

 //mark image as having been modified and hence in the tmp folder  
 needsSaving = true;  

 //deallocate qimage objects
 deallocateLargeImages();
}
//==============================================
void Photo::flipVertically()
{
  //load up image
  //if not modified load from permanent location
  fullImage = new QImage(imageLocation);
  
  //create new image
  QImage* rotatedImage = new QImage(fullImage->width(),
                                    fullImage->height(),
                                    fullImage->depth());
  rotatedImage->setAlphaBuffer(true);
  //copy data                                    
  int x,y;
  for(x=0; x < fullImage->width(); x++)
  {
    for(y=0; y < fullImage->height(); y++)
    {
      rotatedImage->setPixel(fullImage->width() - 1 - x, y, fullImage->pixel(x, y) );
    }
  }                                    

  //delete old image and set new pointer
  delete fullImage;
  fullImage = rotatedImage;
  
  //create thumbnail
  createImages(fullImage,
                     &slideshowImage,
                     &thumbnailImage,
                     &paddedThumbnailImage,
                     slideshowWidth,
                     slideshowHeight); 
  
  //save out updated full and slideshow images to temp folder and deallocate memory
  imageLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
  slideshowLocation = QDir::homeDirPath() + QString("/.albumShaper/tmp/%1_%2_slideshow.jpg")
                                                            .arg(initialSubalbumNumber)
                                                            .arg(initialNumber);
         
  fullImage->save( imageLocation, "JPEG", 100);
  slideshowImage->save( slideshowLocation, "JPEG", 100);
 

 //mark image as having been modified and hence in the tmp folder  
 needsSaving = true;  

 //deallocate qimage objects
 deallocateLargeImages();
}
//==============================================
void Photo::setThumbnailChecksum(QString val)
{
  thumbnailChecksum = val;
}  
//==============================================
void Photo::setSlideshowChecksum(QString val)
{
  slideshowChecksum = val;
}  
//==============================================
void Photo::setImageChecksum(QString val)
{
  imageChecksum = val;
}  
//==============================================
QString Photo::getThumbnailChecksum()
{
  return thumbnailChecksum;
}  
//==============================================  
QString Photo::getSlideshowChecksum()
{
  return slideshowChecksum;
}  
//==============================================  
QString Photo::getImageChecksum()
{
  return imageChecksum;
}  
//==============================================
void Photo::deallocateLargeImages()
{
  delete fullImage;
  fullImage = NULL;
  
  delete slideshowImage;
  slideshowImage = NULL;
}
//==============================================
void Photo::setImageFilename(QString val)
{
  imageLocation = val;
}
//==============================================
void Photo::setSlideshowFilename(QString val)
{
  slideshowLocation = val;
}
//==============================================
QString Photo::getImageFilename()
{
  return imageLocation;
}
//==============================================
QString Photo::getSlideshowFilename()
{
  return slideshowLocation;
}
//==============================================
void Photo::setNeedsSavingVal(bool val)
{
  needsSaving = val;
}
//==============================================
bool Photo::getNeedsSavingVal()
{
  return needsSaving;
}
//==============================================
int Photo::getInitialPhotoNumber()
{
  return initialNumber;
}
//==============================================
int Photo::getInitialSubalbumNumber()
{
  return initialSubalbumNumber;
}
//==============================================
void Photo::setInitialPhotoNumber(int val)
{
 initialNumber = val; 
}
//==============================================
void Photo::setInitialSubalbumNumber(int val)
{
 initialSubalbumNumber = val; 
}
//==============================================
int Photo::actualSlideshowWidth()
{
  return slideshowWidth;;
}
//==============================================
int Photo::actualSlideshowHeight()
{
  return slideshowHeight;
}
//==============================================
void Photo::setActualSlideshowDimensions(int w, int h)
{
  slideshowWidth = w;
  slideshowHeight = h;
}
//==============================================
