//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qlayout.h>
#include <qlabel.h>
#include <qfont.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qlineedit.h>
#include <qcursor.h>
#include <qapplication.h>
#include <math.h>

//Projectwide includes
#include "photoEditWidget.h"
#include "photoViewWidget.h"
#include "questionDialog.h"
#include "../backend/photo.h"
#include "layoutWidget.h"
#include "../config.h"
#include "../backend/imageTools.h"

#include <iostream.h>
//==============================================
PhotoEditWidget::PhotoEditWidget(QWidget *parent, 
                                                const char* name ) : 
                                                QWidget(parent,name)
{
  //set photo pointer to null by default
  photo = NULL;
  
  //backup image null by default
  originalImage = NULL;
  
  //store layout pointer
  layout = (LayoutWidget*)parent;
  
  //create photo description labels
  photoDescription = new QLabel( this );
  photoDescription->setText( "Photo Description:" );
  photoDescription->setFont( QFont( "Times", 12, QFont::Bold ) );
  photoDescriptionVal = new QLineEdit( this );
  photoDescriptionVal->setFont( QFont( "Times", 12, QFont::Bold ) );
  connect( photoDescriptionVal, SIGNAL(textChanged(const QString&)),
           SLOT(updateDescription(const QString&)) );

  //create photo label
  photoView = new PhotoViewWidget( this, "photo view" );
  
  //create buttons  
  buttons = new QFrame(this);
    
  adjExposureImage = new QPixmap( QString(IMAGE_PATH)+"adjExposure.png" );
  adjExposureButton = new QPushButton( buttons );
  adjExposureButton->setPixmap( *adjExposureImage );
  adjExposureButton->setEnabled(false);
  connect( adjExposureButton, SIGNAL(clicked()), SLOT(adjustExposure()) );
  
  cropImage = new QPixmap( QString(IMAGE_PATH)+"crop.png" );
  cropButton = new QPushButton( buttons );
  cropButton->setPixmap( *cropImage );
  connect( cropButton, SIGNAL(clicked()), SLOT(cropToRegion()) );

  invertImage = new QPixmap( QString(IMAGE_PATH)+"invert.png" );
  invertButton = new QPushButton( buttons );
  invertButton->setPixmap( *invertImage );
  connect( invertButton, SIGNAL(clicked()), SLOT(invertSelection()) );
  
  redEyeImage = new QPixmap( QString(IMAGE_PATH)+"removeRedEye.png" );
  redEyeButton = new QPushButton( buttons );
  redEyeButton->setPixmap( *redEyeImage );
  redEyeButton->setEnabled(false);
  connect( redEyeButton, SIGNAL(clicked()), SLOT(reduceRedeye()) );
  
  resetImage = new QPixmap( QString(IMAGE_PATH)+"reset.png" );
  resetButton = new QPushButton( buttons );
  resetButton->setPixmap( *resetImage );
  connect( resetButton, SIGNAL(clicked()), SLOT(resetImageAction()) );
    
  returnImage = new QPixmap( QString(IMAGE_PATH)+"return.png" );
  returnButton = new QPushButton( buttons );
  returnButton->setPixmap( *returnImage );
  connect( returnButton, SIGNAL(clicked()), SLOT(returnFromEdit()) );
  
  //place all items in grid layout
  grid = new QGridLayout( this, 3, 3, 0 );
  grid->addWidget( photoDescription, 0, 0, Qt::AlignLeft );
  grid->addWidget( photoDescriptionVal, 0, 1 );
  grid->addMultiCellWidget( photoView, 1, 1, 0, 2, Qt::AlignCenter );

  grid2 = new QGridLayout( buttons, 1, 8, 0 );
  grid2->addWidget( adjExposureButton, 0, 1, Qt::AlignLeft );
  grid2->addWidget( cropButton, 0, 2, Qt::AlignLeft );
  grid2->addWidget( invertButton, 0, 3, Qt::AlignLeft );
  grid2->addWidget( redEyeButton, 0, 4, Qt::AlignLeft );
  grid2->addWidget( resetButton, 0, 5, Qt::AlignLeft );
  grid2->addWidget( returnButton, 0, 6, Qt::AlignLeft );
  
  //set first and last column of button grid to take up remaining space
  grid2->setColStretch( 0, 1 );
  grid2->setColStretch( 7, 1 );
  grid->addMultiCellWidget( buttons, 2, 2, 0, 2 );
  
  //set the last column to stretch to fill any blank space
  grid->setColStretch( 2, 1 );

  //set photo row to stretch to fill any blank space
  grid->setRowStretch( 1, 1 );
   
  //Set the second column, the actual photo description
  //to have a minimum width
  grid->addColSpacing(1, 300 );
  
  //set the background of the widget to be white
  setPaletteBackgroundColor( QColor(255, 255, 255) );
}
//==============================================
PhotoEditWidget::~PhotoEditWidget()
{
  delete originalImage;
  originalImage = NULL;
}
//==============================================
void PhotoEditWidget::setPhoto(Photo* photo)
{
  //store photo object handle
  this->photo = photo;
  
  //delete old original photo
  delete originalImage;
  originalImage = NULL;
  
  //set description
  photoDescriptionVal->setText( photo->getDescription() );

  //update view of photo
  photoView->setPhoto(photo);

  //check to see if slideshow dimensions are known, 
  //if not load image now and compute size, set
  //dimensions in photo object for future use
  if(photo->actualSlideshowWidth() == -1)
  {
    QImage *backendImage = new QImage( photo->getImageFilename() );
    
    int w, h;   
    resizeImage( backendImage->width(), backendImage->height(),
                      SLIDESHOW_WIDTH, SLIDESHOW_HEIGHT,
                      w, h);
    photo->setActualSlideshowDimensions( w, h);    
    delete backendImage;
    backendImage = NULL;
  }
}
//==============================================
void PhotoEditWidget::updateDescription( const QString& val )
{
  if(photo != NULL)
    photo->setDescription(val);
}
//==============================================
void PhotoEditWidget::adjustExposure()
{
  //---------------------------------
  //load the full image
  QImage *backendImage = new QImage( photo->getImageFilename() );
  //---------------------------------------------
  //find mean and max log of luminance
  double delta = 0.00001;
  double max = 0.0;
  double sum = 0.0;
  double tmp, tmp2;
  int x;
  for(x=0; x<backendImage->width(); x++)
  {
    int y;
    for(y=0; y<backendImage->height(); y++)
    {
      QRgb val = backendImage->pixel( x, y );
      tmp = qGray(val);
      tmp = tmp / 255.0;
      tmp2 = log( tmp + delta );
      
      sum+=tmp2;
      if(tmp > max)
        max = tmp;
    }
  }
  double avg = exp(sum/(backendImage->width() * backendImage->height()));
  cout << "max: " << max << endl;
  cout << "avg: " << avg << endl;
  double whiteSquared = max*max;
  //---------------------------------------------
  //create new image and fill in adjusted values
  QImage* equalizedImage = new QImage(backendImage->width(),
                                                            backendImage->height(),
                                                            backendImage->depth());
  double r, g, b;
  double alpha = 0.18;
  for(x=0; x<backendImage->width(); x++)
  {
    int y;
    for(y=0; y<backendImage->height(); y++)
    {        
        QRgb val = backendImage->pixel( x, y );
        r = qRed(val); g = qGreen(val); b = qBlue(val);
        r = r/255.0; g = g/255.0; b = b/255.0;
        
        r = (alpha/avg) * r;
        g = (alpha/avg) * g;
        b = (alpha/avg) * b;

        r = (r * (1 + (r / whiteSquared))) / (1 + r);
        g = (g * (1 + (g / whiteSquared))) / (1 + g);
        b = (b * (1 + (b / whiteSquared))) / (1 + b);
                
        r = r*255; g = g*255; b = b*255;
        val = qRgb( (int)r, (int)g, (int)b );
        
        equalizedImage->setPixel( x, y, val );
    }
  }
  //---------------------------------
  ///If this is first manipulation to image then backup original, otherwise
  ///delete intermediate object
  if(originalImage == NULL)
  {
    originalImage = backendImage;
  }
  else
  {
    delete backendImage;
    backendImage = NULL;
  }
  //---------------------------------  
  ///Set new image pointer and create all image forms
  photo->setImage(equalizedImage);
  
  //set image function deletes object once done so remove pointer to memory
  //we don't own any more
  equalizedImage = NULL;
  //---------------------------------
  ///Reload photo view
  photoView->setPhoto( photo );
  //---------------------------------
}
//==============================================
void PhotoEditWidget::cropToRegion()
{
  //---------------------------------
  //first get selected coordinates
  QPoint topLeft, bottomRight;
  photoView->getSelection(topLeft, bottomRight);
  
  //if selection has no width or height then bail immediately
  if(bottomRight.x() - topLeft.x() == 0 ||
     bottomRight.y() - topLeft.y() == 0)
   {
     //scold user for not selection part of the image
     
     return;
   }
  //---------------------------------
  //load the full image
  QImage *backendImage = new QImage( photo->getImageFilename() );
  //---------------------------------  
  //scale the selected coordinates up into actual full size coordinates
  float widthRatio = (1.0f * backendImage->width()) / photo->actualSlideshowWidth();
  float heightRatio = (1.0f * backendImage->height()) / photo->actualSlideshowHeight();
   
  //if user selected to end of photo fix selection to end rather
  //then scale and get numerical error which prevents selection
  //from running to edge ot photo
  if(bottomRight.x() == (photo->actualSlideshowWidth() - 1))
    bottomRight.setX( backendImage->width() - 1 );
  else
    bottomRight.setX( bottomRight.x() * widthRatio );
  
  if(bottomRight.y() == (photo->actualSlideshowHeight() - 1))
    bottomRight.setY( backendImage->height() - 1 );
  else
    bottomRight.setY( bottomRight.y() * heightRatio );
  //---------------------------------        
  //create new fullsize image
  QImage* croppedImage = new QImage(bottomRight.x() - topLeft.x() + 1,
                                                        bottomRight.y() - topLeft.y() + 1,
                                                        backendImage->depth());
  int newX, newY;
  newX = 0;
  
  int x;
  for(x=topLeft.x(); x<= bottomRight.x(); x++)
  {
    newY = 0;
    int y;
    for(y=topLeft.y(); y<= bottomRight.y(); y++)
    {
      croppedImage->setPixel( newX, newY, backendImage->pixel(x, y) );
      newY++;
    }
    newX++;
  }
  //---------------------------------
  ///If this is first manipulation to image then backup original, otherwise
  ///delete intermediate object
  if(originalImage == NULL)
  {
    originalImage = backendImage;
  }
  else
  {
    delete backendImage;
    backendImage = NULL;
  }
  //---------------------------------  
  ///Set new image pointer and create all image forms
  photo->setImage(croppedImage);
  
  //set image function deletes object once done so remove pointer to memory
  //we don't own any more
  croppedImage = NULL;
  //---------------------------------
  ///Reload photo view
  photoView->setPhoto( photo );
  //---------------------------------
}
//==============================================
void PhotoEditWidget::invertSelection()
{
  //---------------------------------
  //first get selected coordinates
  QPoint topLeft, bottomRight;
  photoView->getSelection(topLeft, bottomRight);
  //---------------------------------
  //load the full image
  QImage *backendImage = new QImage( photo->getImageFilename() );
  //---------------------------------  
  //if selected region is empty then invert entire image
  if(bottomRight.x() - topLeft.x() == 0 ||
     bottomRight.y() - topLeft.y() == 0)
  {
    topLeft.setX(0);
    topLeft.setY(0);
    bottomRight.setX(backendImage->width() - 1);
    bottomRight.setY(backendImage->height() - 1);
  }
  //else scale the selected coordinates up into actual full size coordinates
  else
  {
    float widthRatio = (1.0f * backendImage->width()) / photo->actualSlideshowWidth();
    float heightRatio = (1.0f * backendImage->height()) / photo->actualSlideshowHeight();
   
    topLeft.setX( topLeft.x() * widthRatio );
    topLeft.setY( topLeft.y() * heightRatio );
    
    //if user selected to end of photo fix selection to end rather
    //then scale and get numerical error which prevents selection
    //from running to edge ot photo
    if(bottomRight.x() == (photo->actualSlideshowWidth() - 1))
      bottomRight.setX( backendImage->width() - 1 );
    else
      bottomRight.setX( bottomRight.x() * widthRatio );
    
    if(bottomRight.y() == (photo->actualSlideshowHeight() - 1))
      bottomRight.setY( backendImage->height() - 1 );
    else
      bottomRight.setY( bottomRight.y() * heightRatio );
  }
  //---------------------------------        
  //invert image
  QImage* invertedImage = new QImage(backendImage->width(),
                                     backendImage->height(),
                                     backendImage->depth());
  int x;
  for(x=0; x<backendImage->width(); x++)
  {
    int y;
    for(y=0; y<backendImage->height(); y++)
    {
      //if within selected region invert color
      if( x >= topLeft.x() && x <= bottomRight.x() &&
          y >= topLeft.y() && y <= bottomRight.y())
      {
        QRgb val = backendImage->pixel( x, y );
        val = qRgb( 255 - qRed(val), 255 - qGreen(val), 255 - qBlue(val) );
        invertedImage->setPixel( x, y, val );
      }
      //otherwise copy color
      else
      {
        invertedImage->setPixel( x, y, backendImage->pixel(x, y) );
      }
    }
  }
  //---------------------------------
  ///If this is first manipulation to image then backup original, otherwise
  ///delete intermediate object
  if(originalImage == NULL)
  {
    originalImage = backendImage;
  }
  else
  {
    delete backendImage;
    backendImage = NULL;
  }
  //---------------------------------  
  ///Set new image pointer and create all image forms
  photo->setImage(invertedImage);
  
  //set image function deletes object once done so remove pointer to memory
  //we don't own any more
  invertedImage = NULL;
  //---------------------------------
  ///Reload photo view
  photoView->setPhoto( photo );
  //---------------------------------
}
//==============================================
void PhotoEditWidget::reduceRedeye()
{

}
//==============================================
void PhotoEditWidget::resetImageAction()
{
  //if previous state saved revert all changes
  if(originalImage != NULL)
  {
    ///Set new image pointer and create all image forms
    photo->setImage(originalImage);
  
    //set image function deletes object once done so remove pointer to memory
    //we don't own any more
    originalImage = NULL;
  
    ///Reload photo view
    photoView->setPhoto( photo );
  } 
}
//==============================================
void PhotoEditWidget::returnFromEdit()
{
  //no changes? then just exit
  if(originalImage == NULL)
  {
    layout->stopEdit(true);
    return;
  }
  
  //otherwise ask user if they are sure they want to permantly scare their photo :)
  QuestionDialog sure( "Keep changes?",
                       "Once applied alterations cannot be undone.",
                       "warning.png",
                       this );
  //apply changes and exit
  if(sure.exec())  
  {
    layout->stopEdit(true);
  }
  //revert before exiting
  else
  {
    resetImageAction();
    layout->stopEdit(true);
  }
}
//==============================================
