//==============================================
//  copyright            : (C) 2003 by Will Stokes
//==============================================
//  This program is free software; you can redistribute it 
//  and/or modify it under the terms of the GNU General 
//  Public License as published by the Free Software 
//  Foundation; either version 2 of the License, or  
//  (at your option) any later version.         
//
//  As a special exception, Will Stokes gives permission to 
//  link this program with Qt non-commercial edition, and 
//  distribute the resulting executable, without including the 
//  source code for the Qt non-commercial edition in the 
//  source distribution. 
//==============================================

//Systemwide includes
#include <qpixmap.h>
#include <qpainter.h>
#include <qevent.h>
#include <qpoint.h>
#include <qaccel.h>

//Projectwide includes
#include "photoViewWidget.h"
#include "../backend/photo.h"

#include <iostream.h>
//==============================================
PhotoViewWidget::PhotoViewWidget(QWidget *parent, 
                                                   const char* name ) : 
                                                   QWidget(parent,name)
{
  photo = NULL;
  setPaletteBackgroundColor( QColor(255, 255, 255) );
  setMinimumSize(600, 400);
  
  //set default selection off screen
  corner1 = new QPoint(-1, -1);
  corner2 = new QPoint(-1, -1);
  
  //by default not in drag mode
  currentlyDragging = false;
  
  //no photo set yet
  photoObject = NULL;
  
  QAccel *keyAccel = new QAccel( this );
  keyAccel->connectItem( keyAccel->insertItem( CTRL + Key_A),
                                      this, SLOT(selectAll()) );
}
//==============================================
PhotoViewWidget::~PhotoViewWidget()
{
  delete photo;
  photo = NULL;
}
//==============================================
void PhotoViewWidget::setPhoto(Photo* phto)
{
  //delete pixmap
  delete photo;
  
  //construct new pixmap
  photo = new QPixmap(phto->getSlideshowFilename());  
  
  //save photo object handle
  photoObject = phto;
  
  //reset selection area to nothing
  corner1->setX(-1); 
  corner1->setY(-1);
  corner2->setX(-1); 
  corner2->setY(-1);
  
  //repaint
  repaint(false);
}
//==============================================
void PhotoViewWidget::paintEvent(QPaintEvent *e)
{  
    //if no photo just paint white
    if(photo == NULL)
    {
      QPainter p;
      p.begin(this);
      p.fillRect(e->rect(), Qt::white);
      p.end();
      return;
    }
    
    //create buffer to draw in
    QRect rct = rect();
    rct.moveBy(-x(), -y());
    QPixmap buffer( size() );
    
    //create a painter pointing to the buffer
    QPainter bufferPainter( &buffer );

    //first flood entire region with white
    buffer.fill( white );
    
    //next paint the image
    QRect pr(0, 0, photo->width(), photo->height());
    bufferPainter.drawPixmap(pr, *photo);
    
    //next draw selection rectangle
    int xmin, xmax, ymin, ymax;
    
    if(corner1->x() < corner2->x())
      xmin = corner1->x();
    else
      xmin = corner2->x();
    if(corner1->x() > corner2->x())
      xmax = corner1->x();
    else
      xmax = corner2->x();
    if(corner1->y() < corner2->y())
      ymin = corner1->y();
    else
      ymin = corner2->y();
    if(corner1->y() > corner2->y())
      ymax = corner1->y();
    else
      ymax = corner2->y();
      
    QRect selection( xmin, ymin, xmax-xmin+1, ymax-ymin+1 );
    
    //construct a pen for selection drawing purposes
    QPen selectionPen;

    //draw selection in white    
    selectionPen.setStyle( Qt::SolidLine );
    selectionPen.setColor( white );
    bufferPainter.setPen( selectionPen);
    bufferPainter.drawRect(selection); 
    
    //draw selection a second time with dashed black, thus 
    //producing an alternating black-white selection rectangle
    selectionPen.setStyle( Qt::DotLine );
    selectionPen.setColor( black );
    bufferPainter.setPen( selectionPen);
    bufferPainter.drawRect(selection); 
    
    bufferPainter.end();
    
    //paint buffer to widget
    bitBlt( this, e->rect().topLeft(), &buffer, e->rect() );
}
//==============================================
void PhotoViewWidget::keyReleaseEvent( QKeyEvent *e)
{
  cout << "Key Pressed: " << e->ascii() << endl;
}
//==============================================
void PhotoViewWidget::mousePressEvent(QMouseEvent *e)
{
  if(photo == NULL)
    return;
    
  currentlyDragging = true;
  QPoint p = cropSelectedPoint(e->pos());
  corner1->setX( p.x() );
  corner1->setY( p.y() );
  corner2->setX( corner1->x() );
  corner2->setY( corner1->y() );
  repaint(false);
}
//==============================================
void PhotoViewWidget::mouseMoveEvent(QMouseEvent *e)
{
  if(!currentlyDragging)
    return;
    
  QPoint p = cropSelectedPoint(e->pos());
  corner2->setX( p.x() );
  corner2->setY( p.y() );
  
  repaint(false);
}
//==============================================
void PhotoViewWidget::mouseReleaseEvent(QMouseEvent *)
{
  currentlyDragging = false;
}
//==============================================
QPoint PhotoViewWidget::cropSelectedPoint(QPoint p)
{
  int newX = p.x();
  int newY = p.y();
  
  int minXAllow, maxXAllow, minYAllow, maxYAllow;
  int xdiff, ydiff;
    
  xdiff = width() - photoObject->actualSlideshowWidth();
  ydiff = height() - photoObject->actualSlideshowHeight();

  minXAllow = xdiff/2;
  minYAllow = ydiff/2;
  
  maxXAllow = minXAllow + photoObject->actualSlideshowWidth() - 1;
  maxYAllow = minYAllow + photoObject->actualSlideshowHeight() - 1;

  if(newX < minXAllow)
    newX = minXAllow;
  if(newX > maxXAllow)
    newX = maxXAllow;
  if(newY < minYAllow)
    newY = minYAllow;
  if(newY > maxYAllow)
    newY = maxYAllow;
  
  return QPoint(newX, newY);
}
//==============================================
void PhotoViewWidget::selectAll()
{
  int minXAllow, maxXAllow, minYAllow, maxYAllow;
  int xdiff, ydiff;
    
  xdiff = width() - photoObject->actualSlideshowWidth();
  ydiff = height() - photoObject->actualSlideshowHeight();

  minXAllow = xdiff/2;
  minYAllow = ydiff/2;  
  maxXAllow = minXAllow + photoObject->actualSlideshowWidth() - 1;
  maxYAllow = minYAllow + photoObject->actualSlideshowHeight() - 1;

  corner1->setX( minXAllow );
  corner1->setY( minYAllow );
  corner2->setX( maxXAllow );
  corner2->setY( maxYAllow );

  repaint(false);
}
//==============================================
void PhotoViewWidget::getSelection(QPoint &topLeft, QPoint &bottomRight)
{
  //if none selected just return that
  if(corner1->x() == -1)
  {
    topLeft.setX(corner1->x());
    topLeft.setY(corner1->y());
    bottomRight.setX(corner2->x());
    bottomRight.setY(corner2->y());
    return;
  }

  //--
  //set coordinates based on raw selection
  //--
  if(corner1->x() < corner2->x())
  {
    topLeft.setX( corner1->x() );
    bottomRight.setX( corner2->x() );
  }
  else
  {
    topLeft.setX( corner2->x() );
    bottomRight.setX( corner1->x() );
  }
  //--
  if(corner1->y() < corner2->y())
  {
    topLeft.setY( corner1->y() );
    bottomRight.setY( corner2->y() );
  }
  else
  {
    topLeft.setY( corner2->y() );
    bottomRight.setY( corner1->y() );
  }
  //--
  //subtract offsets from coordinates
  int xbuff = (width() - photoObject->actualSlideshowWidth()) / 2;
  int ybuff = (height() - photoObject->actualSlideshowHeight()) / 2;
  topLeft.setX( topLeft.x() - xbuff );
  topLeft.setY( topLeft.y() - ybuff );
  bottomRight.setX( bottomRight.x() - xbuff );
  bottomRight.setY( bottomRight.y() - ybuff );
  //--
}
//==============================================
